# (c) Copyright 2006-2008. CodeWeavers, Inc.
package CXSunpkg;
use strict;
use CXLog;
use CXUtils;



#####
#
# Static functions
#
#####

sub pkg_sum($)
{
    my ($filename)=@_;
    my $fh;
    if (!open($fh, "<", $filename))
    {
        cxerr("unable to open '$filename' for reading: $!\n");
        return "";
    }

    my $rs=$/;
    $/=undef;
    my $checksum = unpack("%32C*", <$fh>) % 65535;
    close($fh);
    $/=$rs;
    return $checksum;
}

sub compute_package_name($$)
{
    my ($prefix, $name)=@_;

    # Some Solaris 8 and older versions have a 9 character limit on
    # package names!
    my $max=9-length($prefix);

    my $pkg=$name;
    $pkg =~ s/^managed_//;
    if (length($pkg) > $max or $pkg =~ /^clientp?$/)
    {
        $pkg=substr($pkg, 0, $max-2) . CXUtils::base32(CXUtils::hash_string($name), 2);
    }
    return "$prefix$pkg";
}


#####
#
# The CXSunpkg class
#
#####

sub new($$)
{
    my ($class, $pkgdir)=@_;

    if (!-d "$pkgdir/install" and !cxmkpath("$pkgdir/install", 0755))
    {
        cxerr("unable to create the '$pkgdir/install' directory: $@\n");
        return;
    }

    my $self={pkgmap => ["13"],
              pkgdir => $pkgdir
             };

    bless $self, $class;
    return $self;
}

sub ignore_directory($$$)
{
    #my ($self, $root, $dir)=@_;
    return 0;
}

sub ignore_file($$$)
{
    #my ($self, $root, $file)=@_;
    return 0;
}

sub add_symlink($$$)
{
    my ($self, $rlink, $rdst)=@_;
    $rlink=~s!^reloc/!!;
    push @{$self->{pkgmap}}, "1 s none $rlink=$rdst";
    return 1;
}

sub add_file($$$;$)
{
    my ($self, $type, $rfile, $src)=@_;
    my $pkgmap=$self->{pkgmap};

    my $dst="$self->{pkgdir}/$rfile";
    if (defined $src)
    {
        # FIXME: Preserve the mode?
        require File::Copy;
        if (!File::Copy::copy($src, $dst))
        {
            cxerr("unable to copy '$src' to '$dst': $!\n");
            return 0;
        }
    }

    my ($size, $mtime)=(stat($dst))[7,9];
    my $chksum=pkg_sum($dst);
    if ($type eq "i")
    {
        $rfile =~ s!install/!!;
        push @$pkgmap, "1 i $rfile $size $chksum $mtime";
    }
    else
    {
        $rfile=~s!^reloc/!!;
        my $mode=sprintf "%04o", (stat($dst))[2] & 07777;
        push @$pkgmap, "1 f none $rfile $mode root bin $size $chksum $mtime";
    }
    @$pkgmap[0]+=1+($size >> 9);
    return 1;
}

sub add_dir($$$)
{
    my ($self, $rdir, $mode)=@_;
    my $pkgmap=$self->{pkgmap};

    my $dir="$self->{pkgdir}/$rdir";
    if (!-d $dir and !cxmkpath($dir, $mode))
    {
        cxerr("unable to create the '$dir' directory: $@\n");
        return 0;
    }

    $rdir=~s!^reloc/!!;
    $mode=sprintf "%04o", $mode;
    # FIXME: Should probably explicitly convert to octal
    push @{$self->{pkgmap}}, "1 d none $rdir $mode root bin";
    return 1;
}

sub add_tree($$$)
{
    my ($self, $rdir, $srcdir)=@_;

    my @dirs=("", 0755);
    while (@dirs)
    {
        my $dir=shift @dirs;
        my $mode=shift @dirs;
        cxlog("dir=$dir\n");
        return 0 if (!$self->add_dir("$rdir$dir", $mode));

        my $dh;
        if (!opendir($dh, "$srcdir/$dir"))
        {
            cxerr("unable to open the '$srcdir/$dir' directory: $!\n");
            return 0;
        }
        my $rc=1;
        foreach my $dentry (readdir $dh)
        {
            next if ($dentry =~ /^\.\.?$/);
            $dentry="$dir/$dentry";

            if (-l "$srcdir/$dentry")
            {
                next if ($self->ignore_file($srcdir, $dentry));
                my $dst=readlink "$srcdir$dentry";
                $rc=$self->add_symlink("$rdir$dentry", $dst);
            }
            elsif (-d _)
            {
                $mode=(stat(_))[2] & 07777;
                next if ($self->ignore_directory($srcdir, $dentry));
                push @dirs, $dentry, $mode;
            }
            else
            {
                next if ($self->ignore_file($srcdir, $dentry));
                $rc=$self->add_file("f", "$rdir$dentry", "$srcdir$dentry");
            }
            last if (!$rc);

            if ($dentry =~ /[ =]/)
            {
                $dentry=~ s!^/!!;
                cxerr("cannot package '$dentry'\n");
                cxerr("Solaris packages cannot handle paths containing spaces or equal signs\n");
                return 0;
            }
        }
        closedir($dh);
        return 0 if (!$rc);
    }
    return 1;
}

sub finalize($)
{
    my ($self)=@_;
    my $pkgmap=$self->{pkgmap};
    @$pkgmap[0]=": 1 @$pkgmap[0]";

    my $fh;
    if (!open($fh, ">", "$self->{pkgdir}/pkgmap"))
    {
        cxerr("unable to open '$self->{pkgdir}/pkgmap' for writing: $!\n");
        return 0;
    }
    print $fh join("\n", @$pkgmap), "\n";
    close($fh);
    return 1;
}

return 1;
